#!/bin/bash

INSTALL_PATH=/usr/whatap/whatap-npmd
UBUNTU_SUPPORT_KERNEL_VERSION="5.4"
CENTOS_SUPPORT_KERNEL_VERSION="4.18"
NPM_PATH=/usr/whatap/whatap-npmd/whatap_npmd
TAGRULE_PATH=/usr/whatap/whatap-npmd/tagRule.yaml
DEFAULT_TAGRULE_PATH=/usr/whatap/whatap-npmd/defaultTagRule.yaml
BTF_FILE=/sys/kernel/btf/vmlinux

OS_ID=unknown
OS_VERSION=unknown
CPU=unknown

BPF_SUPPORT=1
accesskey=$1
host=$2
packet=$3

# 0단계: 권한 체크
if [ "$EUID" != "0" ]; then
	echo "This script must be run as root or with sudo" >&2
	exit
fi

# 커널 & OS 버전 체크
function check_kernel_version {
    # Get kernel version
    SUPPORT_KERNEL_VERSION=5.4

    if [ -f "/etc/os-release" ]; then
        source /etc/os-release
        if [[ $ID == "rhel" || $ID_LIKE == *"rhel"* ]]; then
            SUPPORT_KERNEL_VERSION=4.18
        elif [[ $ID == "ubuntu" ]]; then
            OS_ID=ubuntu
            OS_VERSION=$VERSION_ID
        fi
    fi

	KERNEL_VERSION=$(uname -r | cut -d '.' -f 1-2)
	if [[ "$(echo -e "$KERNEL_VERSION\n$SUPPORT_KERNEL_VERSION" | sort -V | head -n1)" == "$SUPPORT_KERNEL_VERSION" ]]; then
		echo "Supported eBPF Mode : $ID, $ID_LIKE, $KERNEL_VERSION. OK."
	else
		echo "Unsupported eBPF Mode, Requires a $SUPPORT_KERNEL_VERSION or higher."
        if [ "$packet" == "1" ]; then
            BPF_SUPPORT=0
        else 
            exit
        fi
	fi
}

# CPU 아키텍처 체크
function check_cpu_arch {
    CPU_ARCH=$(uname -m)
	if [ "$CPU_ARCH" == "x86_64" ]; then
		echo "Supported CPU Arch : $CPU_ARCH, OK."
        CPU=x86_64 
    elif [ "$CPU_ARCH" == "arm64" ] || [ "$CPU_ARCH" == "aarch64" ]; then
		echo "Supported CPU Arch : $CPU_ARCH, OK."
        CPU=arm64
	else
		echo "Unsupported CPU Arch : $CPU_ARCH"
		exit
	fi
}



    
#
function check_btf_file {
    if [ "$BPF_SUPPORT" == "0" ]; then
        BTF_FILE=""
        return
    fi

    KERNEL_VERSION_LONG=$(uname -r)

    if [ -f "$BTF_FILE" ]; then
        echo "BTF File Check Completed.($BTF_FILE)"
        return
    else
        if [ ! -d "$INSTALL_PATH" ]; then
            mkdir -p "$INSTALL_PATH"
        fi

        BTF_NAME=$KERNEL_VERSION_LONG.btf
        BTF_FILE=$INSTALL_PATH/$BTF_NAME
        BTF_FILE_TAR=""
        if [ -f "$BTF_FILE" ]; then
            echo "BTF File Check Completed.($BTF_FILE)"
            return
        else 
            echo "BTF File does not exist."
            echo "Search Package..."
            BTF_PACKAGE_PATH=$INSTALL_PATH/btf/$OS_ID/$OS_VERSION/$CPU/$BTF_NAME.tar.xz
            if [ -f "$BTF_PACKAGE_PATH" ]; then
                echo "Package File Check Ok."
                BTF_FILE_TAR=$BTF_PACKAGE_PATH
            else
                BTF_FILE_TAR=$BTF_FILE.tar.xz
                echo "Search in the Whatap repository..."
                wget -O $BTF_FILE_TAR "http://repo.whatap.io/btf/$OS_ID/$OS_VERSION/$CPU/$BTF_NAME.tar.xz"
                if [[ $? -eq 0 ]]; then
                    echo "Whatap repository Search Ok."
                else
                    echo "Unverified System Environment"
                    echo " OS ID: $OS_ID"
                    echo " OS VERSION: $OS_VERSION"
                    echo " CPU ARCH : $CPU"
                    echo " KERNEL VERSION: $KERNEL_VERSION_LONG"
                    exit
                fi
            fi

            tar -xf $BTF_FILE_TAR -C $INSTALL_PATH
            if [[ $? -eq 0 ]]; then
                echo "BTF File Decompress OK."
            else
                echo "tar command fail"
                exit
            fi

            echo "BTF File Check Completed.($BTF_FILE)"
        fi
    fi
}

# 패키지 파일 이동
function move_package_files {
	# 이 함수에서는 패키지 내의 파일을 지정된 경로로 이동시키는 작업을 수행합니다.
	# 이를 위해 필요한 로직을 추가하세요.
	echo "Move package files"
	# 1. INSTALL_PATH 변수에 저장된 디렉토리의 존재 여부를 확인하고 없으면 새로 만들어줍니다.
	if [ ! -d "$INSTALL_PATH" ]; then
		mkdir -p "$INSTALL_PATH"
	fi

	# 2. CPU_ARCH가 x86_64라면 npmagent_x86_64라는 파일을 whatap_npmd라는 이름으로 바꾼뒤 INSTALL_PATH에 옮깁니다.
	if [ "$CPU_ARCH" == "x86_64" ]; then

        if [ -f "$INSTALL_PATH/npmagent_x86_64" ]; then
            mv -f "$INSTALL_PATH/npmagent_x86_64" "$INSTALL_PATH/whatap_npmd"
        fi
        if [ -f "$INSTALL_PATH/npmagent_arm64" ]; then
            rm -rf $INSTALL_PATH/npmagent_arm64
        fi
	fi

	# 3. CPU_ARCH가 arm64라면 npmagent_arm64라는 파일을 whatap_npmd라는 이름으로 바꾼뒤 INSTALL_PATH에 옮깁니다.
	if [ "$CPU_ARCH" == "arm64" ] || [ "$CPU_ARCH" == "aarch64" ]; then

        if [ -f "$INSTALL_PATH/npmagent_arm64" ]; then
            mv -f "$INSTALL_PATH/npmagent_arm64" "$INSTALL_PATH/whatap_npmd"
        fi
        if [ -f "$INSTALL_PATH/npmagent_x86_64" ]; then
            rm -rf npmagent_x86_64
        fi
	fi
}

# config 생성
function create_config {
	# config 파일 이름
    CONFIG_FILE=$INSTALL_PATH"/whatap.conf"

    if test -f "$CONFIG_FILE"; then
        check=1
    else
        touch $CONFIG_FILE
    fi

	if [ -n "$accesskey" ]; then 
		check=`grep -nir "accesskey" $CONFIG_FILE | wc -l`
		if [ $check -lt 1 ]; then
			echo "accesskey=$accesskey" >> "$CONFIG_FILE"
		fi
	fi

	if [ -n "$host" ]; then 
		check=`grep -nir "whatap.server.host" $CONFIG_FILE | wc -l`
		if [ $check -lt 1 ]; then
			echo "whatap.server.host=$host" >> "$CONFIG_FILE"
		fi
	fi

    PUBLIC_IP=""
    exit_status=1

    if command -v curl &>/dev/null; then
        PUBLIC_IP=`curl -m 1 ifconfig.me`
        exit_status=$?
    elif command -v wget &>/dev/null; then
        PUBLIC_IP=`wget -qO- ifconfig.me`
        exit_status=$?
    fi

	if [[ $exit_status -eq 0 ]]; then
		check=`grep -nir "publicIP" $CONFIG_FILE | wc -l`
		if [ $check -lt 1 ]; then
			echo "publicIP=$PUBLIC_IP" >> "$CONFIG_FILE"
		fi
	fi

    check=`grep -nir "tagRuleConfig" $CONFIG_FILE | wc -l`
    if [ $check -lt 1 ]; then
		echo "tagRuleConfig=/usr/whatap/whatap-npmd/tagRule.yaml" >> $CONFIG_FILE
    fi

    if [ "$BTF_FILE" != "" ]; then
        sed -i "/btfFilePath/d" "$CONFIG_FILE"
        echo "btfFilePath=$BTF_FILE" >> $CONFIG_FILE
    fi
}

function default_tagRule {
    if test -f "$TAGRULE_PATH"; then
        echo "There are already created files. ($TAGRULE_PATH)"
        return
    fi 

    cp -f $DEFAULT_TAGRULE_PATH $TAGRULE_PATH
}


# 서비스 등록 및 시작
function start_service {
    echo "Start service"
    k8s_status=0
    #Kuber는 별도 실행
    if [ -n "$KUBERNETES_SERVICE_HOST" ]; then
	echo "KUBERNETES_SERVICE_HOST=\"$KUBERNETES_SERVICE_HOST\"" |  tee -a /etc/environment > /dev/null
	k8s_status=1
    fi

    if [ -n "$KUBERNETES_SERVICE_PORT" ]; then
	echo "KUBERNETES_SERVICE_PORT=\"$KUBERNETES_SERVICE_PORT\"" |  tee -a /etc/environment > /dev/null
	k8s_status=1
    fi

    if [[ $k8s_status -eq 1 ]]; then
	return
    fi
    if command -v sudo &> /dev/null ; then
        sudo service whatap-npmd restart
    else
        service whatap-npmd restart
    fi
}

# 스크립트 실행
check_kernel_version
check_cpu_arch
check_btf_file
#move_package_files
create_config
default_tagRule
start_service

echo "Installation complete!"

